import uuid

from luthor.models.hearing import Hearing
from sqlalchemy import Boolean, Column, Date, ForeignKey, Unicode, desc, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship
from sqlalchemy.sql import expression

from .meta import Base


class Case(Base):
    __tablename__ = "cases"

    id = Column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), default=uuid.uuid4
    )
    office_file_number = Column("office_file_number", Unicode(255), nullable=False, unique=True)
    court_case_number = Column("court_case_number", Unicode(255), nullable=False, unique=True)
    year = Column("year", Unicode(255), nullable=False, unique=True)
    title = Column("title", Unicode(255), nullable=False, unique=True)
    docket_number = Column("docket_number", Unicode(255), nullable=False, unique=True)
    receipt_date = Column("receipt_date", Date, nullable=False, index=True)
    limitation_date = Column("limitation_date", Date, nullable=False, index=True)
    filing_date = Column("filing_date", Date, nullable=False, index=True)
    appear_on_behalf_of = Column("appear_on_behalf_of", Unicode(255), nullable=False, unique=True)
    question_of_law = Column("question_of_law", Unicode(255), nullable=False, unique=True)
    aor_name = Column("aor_name", Unicode(255), nullable=False, unique=True)
    opposing_council_aor = Column("opposing_council_aor", Unicode(255), nullable=False, unique=True)
    opposing_council_detail = Column("opposing_council_detail", Unicode(255), nullable=False)
    lower_court_case_number = Column("lower_court_case_number", Unicode(255), nullable=False, unique=True)
    date_of_impugned_judgement = Column("date_of_impugned_judgement", Date, nullable=False, index=True)
    brief_description = Column("brief_description", Unicode(255), nullable=False, unique=True)
    remarks = Column("remarks", Unicode(255), nullable=False, unique=True)
    slp_counter = Column("slp_counter", Unicode(255), nullable=False, unique=True)
    contact_detail = Column("contact_detail", Unicode(255), nullable=False, unique=True)
    case_connected_with = Column("case_connected_with", Unicode(255), nullable=False, unique=True)
    bunch_cases = Column("bunch_cases", Unicode(255), nullable=False)
    is_deleted = Column("is_deleted", Boolean, nullable=False, server_default=expression.false(), default=False)

    court_id = Column("court_id", UUID(as_uuid=True), ForeignKey("courts.id"), nullable=True)
    department_id = Column("department_id", UUID(as_uuid=True), ForeignKey("departments.id"), nullable=False)
    office_id = Column("office_id", UUID(as_uuid=True), ForeignKey("offices.id"), nullable=False)
    case_type_id = Column("case_type_id", UUID(as_uuid=True), ForeignKey("case_types.id"), nullable=True)
    act_id = Column("act_id", UUID(as_uuid=True), ForeignKey("acts.id"), nullable=True)
    nature_id = Column("nature_id", UUID(as_uuid=True), ForeignKey("natures.id"), nullable=False)
    office_status_id = Column(
        "office_status_id", UUID(as_uuid=True), ForeignKey("office_statuses.id"), nullable=False
    )  # Old Name Latest Status
    court_status_id = Column(
        "court_status_id", UUID(as_uuid=True), ForeignKey("court_statuses.id"), nullable=True
    )  # Old Name Status In Court
    case_source_id = Column("case_source_id", UUID(as_uuid=True), ForeignKey("case_sources.id"), nullable=True)

    court = relationship("Court", backref="cases")
    department = relationship("Department", backref="cases")
    office = relationship("Office", backref="cases")
    case_type = relationship("CaseType", backref="cases")
    act = relationship("Act", backref="cases")
    nature = relationship("Nature", backref="cases")
    office_status = relationship("OfficeStatus")
    court_status = relationship("CourtStatus", back_populates="cases")
    case_source = relationship("CaseSource", back_populates="cases")
    hearings = relationship("Hearing", order_by=desc(Hearing.next_hearing_date), back_populates="case")

    def __init__(
        self,
        office_file_number=None,
        court_case_number=None,
        year=None,
        title=None,
        docket_number=None,
        receipt_date=None,
        limitation_date=None,
        filing_date=None,
        appear_on_behalf_of=None,
        question_of_law=None,
        aor_name=None,
        opposing_council_aor=None,
        opposing_council_detail=None,
        lower_court_case_number=None,
        date_of_impugned_judgement=None,
        brief_description=None,
        remarks=None,
        slp_counter=None,
        contact_detail=None,
        case_connected_with=None,
        bunch_cases=None,
        court_id=None,
        department_id=None,
        office_id=None,
        case_type_id=None,
        act_id=None,
        nature_id=None,
        office_status_id=None,
        court_status_id=None,
        case_source_id=None,
        is_deleted=None,
        hearings=None,
        id_=None,
    ):
        self.office_file_number = office_file_number
        self.court_case_number = court_case_number
        self.year = year
        self.title = title
        self.docket_number = docket_number
        self.receipt_date = receipt_date
        self.limitation_date = limitation_date
        self.filing_date = filing_date
        self.appear_on_behalf_of = appear_on_behalf_of
        self.question_of_law = question_of_law
        self.aor_name = aor_name
        self.opposing_council_aor = opposing_council_aor
        self.opposing_council_detail = opposing_council_detail
        self.lower_court_case_number = lower_court_case_number
        self.date_of_impugned_judgement = date_of_impugned_judgement
        self.brief_description = brief_description
        self.remarks = remarks
        self.slp_counter = slp_counter
        self.contact_detail = contact_detail
        self.case_connected_with = case_connected_with
        self.bunch_cases = bunch_cases
        self.court_id = court_id
        self.department_id = department_id
        self.office_id = office_id
        self.case_type_id = case_type_id
        self.act_id = act_id
        self.nature_id = nature_id
        self.office_status_id = office_status_id
        self.court_status_id = court_status_id
        self.is_deleted = False if is_deleted is None else is_deleted
        self.case_source_id = case_source_id
        if hearings is not None:
            self.hearings = hearings
        self.id = id_
