import uuid

from typing import List

import luthor.schemas.advocate as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.advocate import Advocate
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.Advocate)
def save(
    data: schemas.AdvocateIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["advocates"]),
) -> schemas.Advocate:
    try:
        item = Advocate(
            name=data.name, mobile=data.mobile, landline=data.landline, address=data.address, email=data.email
        )
        db.add(item)
        db.commit()
        return advocate_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.Advocate)
def update(
    id_: uuid.UUID,
    data: schemas.AdvocateIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["advocates"]),
) -> schemas.Advocate:
    try:
        item: Advocate = db.query(Advocate).filter(Advocate.id == id_).first()
        item.name = data.name
        item.mobile = data.mobile
        item.landline = data.landline
        item.address = data.address
        item.email = data.email
        db.commit()
        return advocate_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.AdvocateBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["advocates"]),
) -> schemas.AdvocateBlank:
    try:
        item: Advocate = db.query(Advocate).filter(Advocate.id == id_).first()
        db.delete(item)
        db.commit()
        return advocate_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.AdvocateBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["advocates"]),
) -> schemas.AdvocateBlank:
    return advocate_blank()


@router.get("/list", response_model=List[schemas.Advocate])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [advocate_info(item) for item in db.query(Advocate).order_by(Advocate.name).all()]


@router.get("/{id_}", response_model=schemas.Advocate)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["advocates"]),
) -> schemas.Advocate:
    item: Advocate = db.query(Advocate).filter(Advocate.id == id_).first()
    return advocate_info(item)


def advocate_info(item: Advocate) -> schemas.Advocate:
    return schemas.Advocate(
        id=item.id, name=item.name, mobile=item.mobile, landline=item.landline, address=item.address, email=item.email
    )


def advocate_blank() -> schemas.AdvocateBlank:
    return schemas.AdvocateBlank(name="", mobile="", landline="", address="", email="")
