import uuid

from typing import List

import luthor.schemas.case_source as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.case_source import CaseSource
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.CaseSource)
def save(
    data: schemas.CaseSourceIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["case-sources"]),
) -> schemas.CaseSource:
    try:
        item = CaseSource(name=data.name, prefix=data.prefix)
        db.add(item)
        db.commit()
        return case_source_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.CaseSource)
def update(
    id_: uuid.UUID,
    data: schemas.CaseSourceIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["case-sources"]),
) -> schemas.CaseSource:
    try:
        item: CaseSource = db.query(CaseSource).filter(CaseSource.id == id_).first()
        item.name = data.name
        item.prefix = data.prefix
        db.commit()
        return case_source_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.CaseSourceBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["case-sources"]),
) -> schemas.CaseSourceBlank:
    try:
        item: CaseSource = db.query(CaseSource).filter(CaseSource.id == id_).first()
        db.delete(item)
        db.commit()
        return case_source_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.CaseSourceBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["case-sources"]),
) -> schemas.CaseSourceBlank:
    return case_source_blank()


@router.get("/list", response_model=List[schemas.CaseSource])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [case_source_info(item) for item in db.query(CaseSource).order_by(CaseSource.name).all()]


@router.get("/{id_}", response_model=schemas.CaseSource)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["case-sources"]),
) -> schemas.CaseSource:
    item: CaseSource = db.query(CaseSource).filter(CaseSource.id == id_).first()
    return case_source_info(item)


def case_source_info(item: CaseSource) -> schemas.CaseSource:
    return schemas.CaseSource(id=item.id, name=item.name, prefix=item.prefix)


def case_source_blank() -> schemas.CaseSourceBlank:
    return schemas.CaseSourceBlank(name="", prefix="")
