import uuid

from typing import List

import luthor.schemas.contact as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.contact import Contact
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.Contact)
def save(
    data: schemas.ContactIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["contacts"]),
) -> schemas.Contact:
    try:
        item = Contact(
            name=data.name,
            mobile=data.mobile,
            landline=data.landline,
            address=data.address,
            email=data.email,
            office_id=data.office.id_ if data.office is not None else None,
            department_id=data.department.id_ if data.department is not None else None,
        )
        db.add(item)
        db.commit()
        return contact_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.Contact)
def update(
    id_: uuid.UUID,
    data: schemas.ContactIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["contacts"]),
) -> schemas.Contact:
    try:
        item: Contact = db.query(Contact).filter(Contact.id == id_).first()
        item.name = data.name
        item.mobile = data.mobile
        item.landline = data.landline
        item.address = data.address
        item.email = data.email
        item.office_id = data.office.id_ if data.office is not None else None
        item.department_id = (data.department.id_ if data.department is not None else None,)
        db.commit()
        return contact_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.ContactBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["contacts"]),
) -> schemas.ContactBlank:
    try:
        item: Contact = db.query(Contact).filter(Contact.id == id_).first()
        if item is None:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Sale Category not found",
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_501_NOT_IMPLEMENTED,
                detail="Sale Category deletion not implemented",
            )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.ContactBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["contacts"]),
) -> schemas.ContactBlank:
    return contact_blank()


@router.get("/list", response_model=List[schemas.Contact])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)) -> List[schemas.Contact]:
    return [contact_info(item) for item in db.query(Contact).order_by(Contact.name).all()]


@router.get("/{id_}", response_model=schemas.Contact)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["contacts"]),
) -> schemas.Contact:
    item: Contact = db.query(Contact).filter(Contact.id == id_).first()
    return contact_info(item)


def contact_info(item: Contact) -> schemas.Contact:
    return schemas.Contact(
        id=item.id,
        name=item.name,
        mobile=item.mobile,
        landline=item.landline,
        address=item.address,
        email=item.email,
        office=schemas.OfficeLink(id=item.office.id, name=item.office.name) if item.office is not None else None,
        department=schemas.DepartmentLink(id=item.department.id, name=item.department.name)
        if item.department is not None
        else None,
    )


def contact_blank() -> schemas.ContactBlank:
    return schemas.ContactBlank(name="", mobile="", landline="", address="", email="")
