import uuid

from typing import List

import luthor.schemas.court as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.court import Court
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.Court)
def save(
    data: schemas.CourtIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["courts"]),
) -> schemas.Court:
    try:
        item = Court(name=data.name)
        db.add(item)
        db.commit()
        return court_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.Court)
def update(
    id_: uuid.UUID,
    data: schemas.CourtIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["courts"]),
) -> schemas.Court:
    try:
        item: Court = db.query(Court).filter(Court.id == id_).first()
        item.name = data.name
        db.commit()
        return court_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.CourtBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["courts"]),
) -> schemas.CourtBlank:
    try:
        item: Court = db.query(Court).filter(Court.id == id_).first()
        db.delete(item)
        db.commit()
        return court_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.CourtBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["courts"]),
) -> schemas.CourtBlank:
    return court_blank()


@router.get("/list", response_model=List[schemas.Court])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [court_info(item) for item in db.query(Court).order_by(Court.name).all()]


@router.get("/{id_}", response_model=schemas.Court)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["courts"]),
) -> schemas.Court:
    item: Court = db.query(Court).filter(Court.id == id_).first()
    return court_info(item)


def court_info(item: Court) -> schemas.Court:
    return schemas.Court(id=item.id, name=item.name)


def court_blank() -> schemas.CourtBlank:
    return schemas.CourtBlank(name="")
