import uuid

from typing import List

import luthor.schemas.court_status as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.court_status import CourtStatus
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.CourtStatus)
def save(
    data: schemas.CourtStatusIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["court-statuses"]),
) -> schemas.CourtStatus:
    try:
        item = CourtStatus(name=data.name)
        db.add(item)
        db.commit()
        return court_status_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.CourtStatus)
def update(
    id_: uuid.UUID,
    data: schemas.CourtStatusIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["court-statuses"]),
) -> schemas.CourtStatus:
    try:
        item: CourtStatus = db.query(CourtStatus).filter(CourtStatus.id == id_).first()
        item.name = data.name
        db.commit()
        return court_status_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.CourtStatusBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["court-statuses"]),
) -> schemas.CourtStatusBlank:
    try:
        item: CourtStatus = db.query(CourtStatus).filter(CourtStatus.id == id_).first()
        db.delete(item)
        db.commit()
        return court_status_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.CourtStatusBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["court-statuses"]),
) -> schemas.CourtStatusBlank:
    return court_status_blank()


@router.get("/list", response_model=List[schemas.CourtStatus])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [court_status_info(item) for item in db.query(CourtStatus).order_by(CourtStatus.name).all()]


@router.get("/{id_}", response_model=schemas.CourtStatus)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["court-statuses"]),
) -> schemas.CourtStatus:
    item: CourtStatus = db.query(CourtStatus).filter(CourtStatus.id == id_).first()
    return court_status_info(item)


def court_status_info(item: CourtStatus) -> schemas.CourtStatus:
    return schemas.CourtStatus(id=item.id, name=item.name)


def court_status_blank() -> schemas.CourtStatusBlank:
    return schemas.CourtStatusBlank(name="")
