import uuid

from typing import List

import luthor.schemas.department as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.department import Department
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.Department)
def save(
    data: schemas.DepartmentIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["departments"]),
) -> schemas.Department:
    try:
        item = Department(name=data.name)
        db.add(item)
        db.commit()
        return department_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.Department)
def update(
    id_: uuid.UUID,
    data: schemas.DepartmentIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["departments"]),
) -> schemas.Department:
    try:
        item: Department = db.query(Department).filter(Department.id == id_).first()
        item.name = data.name
        db.commit()
        return department_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.DepartmentBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["departments"]),
) -> schemas.DepartmentBlank:
    try:
        item: Department = db.query(Department).filter(Department.id == id_).first()
        db.delete(item)
        db.commit()
        return department_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.DepartmentBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["departments"]),
) -> schemas.DepartmentBlank:
    return department_blank()


@router.get("/list", response_model=List[schemas.Department])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [department_info(item) for item in db.query(Department).order_by(Department.name).all()]


@router.get("/{id_}", response_model=schemas.Department)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["departments"]),
) -> schemas.Department:
    item: Department = db.query(Department).filter(Department.id == id_).first()
    return department_info(item)


def department_info(item: Department) -> schemas.Department:
    return schemas.Department(id=item.id, name=item.name)


def department_blank() -> schemas.DepartmentBlank:
    return schemas.DepartmentBlank(name="")
