import uuid

from typing import List

import luthor.schemas.nature as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.nature import Nature
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.Nature)
def save(
    data: schemas.NatureIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["natures"]),
) -> schemas.Nature:
    try:
        item = Nature(name=data.name)
        db.add(item)
        db.commit()
        return nature_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.Nature)
def update(
    id_: uuid.UUID,
    data: schemas.NatureIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["natures"]),
) -> schemas.Nature:
    try:
        item: Nature = db.query(Nature).filter(Nature.id == id_).first()
        item.name = data.name
        db.commit()
        return nature_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.NatureBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["natures"]),
) -> schemas.NatureBlank:
    try:
        item: Nature = db.query(Nature).filter(Nature.id == id_).first()
        db.delete(item)
        db.commit()
        return nature_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.NatureBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["natures"]),
) -> schemas.NatureBlank:
    return nature_blank()


@router.get("/list", response_model=List[schemas.Nature])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [nature_info(item) for item in db.query(Nature).order_by(Nature.name).all()]


@router.get("/{id_}", response_model=schemas.Nature)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["natures"]),
) -> schemas.Nature:
    item: Nature = db.query(Nature).filter(Nature.id == id_).first()
    return nature_info(item)


def nature_info(item: Nature) -> schemas.Nature:
    return schemas.Nature(id=item.id, name=item.name)


def nature_blank() -> schemas.NatureBlank:
    return schemas.NatureBlank(name="")
