import uuid

from typing import List

import luthor.schemas.office as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.office import Office
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.Office)
def save(
    data: schemas.OfficeIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["offices"]),
) -> schemas.Office:
    try:
        item = Office(
            name=data.name,
            address=data.address,
            email=data.email,
            department_id=data.department.id_ if data.department is not None else None,
        )
        db.add(item)
        db.commit()
        return office_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.Office)
def update(
    id_: uuid.UUID,
    data: schemas.OfficeIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["offices"]),
) -> schemas.Office:
    try:
        item: Office = db.query(Office).filter(Office.id == id_).first()
        item.name = data.name
        item.address = data.address
        item.email = data.email
        item.department_id = data.department.id_ if data.department is not None else None
        db.commit()
        return office_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.OfficeBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["offices"]),
) -> schemas.OfficeBlank:
    try:
        item: Office = db.query(Office).filter(Office.id == id_).first()
        if item is None:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Sale Category not found",
            )
        else:
            raise HTTPException(
                status_code=status.HTTP_501_NOT_IMPLEMENTED,
                detail="Sale Category deletion not implemented",
            )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.OfficeBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["offices"]),
) -> schemas.OfficeBlank:
    return office_blank()


@router.get("/of-department", response_model=List[schemas.Office])
def show_list_of_department(db: Session = Depends(get_db), user: UserToken = Depends(get_user)) -> List[schemas.Office]:
    return [
        office_info(item)
        for item in db.query(Office).filter(Office.department_id == None).order_by(Office.name).all()  # noqa: E711
    ]


@router.get("/of-department/{department_id}", response_model=List[schemas.Office])
def show_list_of_department_id(
    department_id: uuid.UUID, db: Session = Depends(get_db), user: UserToken = Depends(get_user)
) -> List[schemas.Office]:
    return [
        office_info(item)
        for item in db.query(Office).filter(Office.department_id == department_id).order_by(Office.name).all()
    ]


@router.get("/list", response_model=List[schemas.Office])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)) -> List[schemas.Office]:
    return [office_info(item) for item in db.query(Office).order_by(Office.name).all()]


@router.get("/{id_}", response_model=schemas.Office)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["offices"]),
) -> schemas.Office:
    item: Office = db.query(Office).filter(Office.id == id_).first()
    return office_info(item)


def office_info(item: Office) -> schemas.Office:
    return schemas.Office(
        id=item.id,
        name=item.name,
        address=item.address,
        email=item.email,
        department=schemas.DepartmentLink(id=item.department.id, name=item.department.name)
        if item.department is not None
        else None,
    )


def office_blank() -> schemas.OfficeBlank:
    return schemas.OfficeBlank(name="", address="", email="")
