import uuid

from typing import List

import luthor.schemas.office_status as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.office_status import OfficeStatus
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.post("", response_model=schemas.OfficeStatus)
def save(
    data: schemas.OfficeStatusIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["office-statuses"]),
) -> schemas.OfficeStatus:
    try:
        item = OfficeStatus(name=data.name)
        db.add(item)
        db.commit()
        return office_status_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.put("/{id_}", response_model=schemas.OfficeStatus)
def update(
    id_: uuid.UUID,
    data: schemas.OfficeStatusIn,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["office-statuses"]),
) -> schemas.OfficeStatus:
    try:
        item: OfficeStatus = db.query(OfficeStatus).filter(OfficeStatus.id == id_).first()
        item.name = data.name
        db.commit()
        return office_status_info(item)
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.delete("/{id_}", response_model=schemas.OfficeStatusBlank)
def delete(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["office-statuses"]),
) -> schemas.OfficeStatusBlank:
    try:
        item: OfficeStatus = db.query(OfficeStatus).filter(OfficeStatus.id == id_).first()
        db.delete(item)
        db.commit()
        return office_status_blank()
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
    except Exception:
        db.rollback()
        raise


@router.get("", response_model=schemas.OfficeStatusBlank)
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["office-statuses"]),
) -> schemas.OfficeStatusBlank:
    return office_status_blank()


@router.get("/list", response_model=List[schemas.OfficeStatus])
def show_list(db: Session = Depends(get_db), user: UserToken = Depends(get_user)):
    return [office_status_info(item) for item in db.query(OfficeStatus).order_by(OfficeStatus.name).all()]


@router.get("/{id_}", response_model=schemas.OfficeStatus)
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["office-statuses"]),
) -> schemas.OfficeStatus:
    item: OfficeStatus = db.query(OfficeStatus).filter(OfficeStatus.id == id_).first()
    return office_status_info(item)


def office_status_info(item: OfficeStatus) -> schemas.OfficeStatus:
    return schemas.OfficeStatus(id=item.id, name=item.name)


def office_status_blank() -> schemas.OfficeStatusBlank:
    return schemas.OfficeStatusBlank(name="")
