import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';

import { Department } from '../../core/department';
import { Office } from '../../core/office';
import { ToasterService } from '../../core/toaster.service';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { OfficeService } from '../office.service';

@Component({
  selector: 'app-office-detail',
  templateUrl: './office-detail.component.html',
  styleUrls: ['./office-detail.component.css'],
})
export class OfficeDetailComponent implements OnInit, AfterViewInit {
  @ViewChild('nameElement', { static: true }) nameElement?: ElementRef;
  form: FormGroup;
  departments: Department[] = [];
  item: Office = new Office();

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private fb: FormBuilder,
    private toaster: ToasterService,
    private ser: OfficeService,
  ) {
    // Create form
    this.form = this.fb.group({
      name: '',
      email: '',
      address: '',
      department: '',
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { item: Office; departments: Department[] };

      this.departments = data.departments;
      this.showItem(data.item);
    });
  }

  showItem(item: Office) {
    this.item = item;
    this.form.setValue({
      name: this.item.name,
      email: this.item.email,
      address: this.item.address,
      department: this.item.department ? this.item.department.id : '',
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.nameElement !== undefined) {
        this.nameElement.nativeElement.focus();
      }
    }, 0);
  }

  save() {
    this.ser.saveOrUpdate(this.getItem()).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/offices');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  delete() {
    this.ser.delete(this.item.id as string).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/offices');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: { title: 'Delete Office?', content: 'Are you sure? This cannot be undone.' },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): Office {
    const formModel = this.form.value;
    this.item.name = formModel.name;
    this.item.email = formModel.email;
    this.item.address = formModel.address;
    if (formModel.department === undefined || formModel.department === '') {
      this.item.department = undefined;
    } else {
      if (this.item.department === null || this.item.department === undefined) {
        this.item.department = new Department();
      }
      this.item.department.id = formModel.department;
    }
    return this.item;
  }
}
