import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';
import { Office } from '../core/office';

const httpOptions = {
  headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
};
const url = '/api/offices';
const serviceName = 'OfficeService';

@Injectable({
  providedIn: 'root',
})
export class OfficeService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string | null): Observable<Office> {
    const getUrl: string = id === null ? url : `${url}/${id}`;
    return this.http
      .get<Office>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<Office>;
  }

  list(): Observable<Office[]> {
    return this.http
      .get<Office[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Office[]>;
  }

  filteredList(departmentId: string): Observable<Office[]> {
    return this.http
      .get<Office[]>(`${url}/of-department/${departmentId}`)
      .pipe(
        catchError(this.log.handleError(serviceName, `filteredList departmentId=${departmentId}`)),
      ) as Observable<Office[]>;
  }

  save(office: Office): Observable<Office> {
    return this.http
      .post<Office>(url, office, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<Office>;
  }

  update(office: Office): Observable<Office> {
    return this.http
      .put<Office>(`${url}/${office.id}`, office, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<Office>;
  }

  saveOrUpdate(office: Office): Observable<Office> {
    if (!office.id) {
      return this.save(office);
    }
    return this.update(office);
  }

  delete(id: string): Observable<Office> {
    return this.http
      .delete<Office>(`${url}/${id}`, httpOptions)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<Office>;
  }
}
