import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { ToasterService } from '../core/toaster.service';
import { ToCsvService } from '../shared/to-csv.service';

import { SaleReport } from './sale-report';
import { SaleReportDatasource } from './sale-report-datasource';
import { SaleReportService } from './sale-report.service';

@Component({
  selector: 'app-sale-report',
  templateUrl: './sale-report.component.html',
  styleUrls: ['./sale-report.component.css'],
})
export class SaleReportComponent implements OnInit {
  info: SaleReport = new SaleReport();
  dataSource: SaleReportDatasource = new SaleReportDatasource(this.info.amounts);
  form: FormGroup;

  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['name', 'amount'];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private fb: FormBuilder,
    private toCsv: ToCsvService,
    private toaster: ToasterService,
    private ser: SaleReportService,
  ) {
    // Create form
    this.form = this.fb.group({
      startDate: '',
      finishDate: '',
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: SaleReport };
      this.info = data.info;
      this.form.setValue({
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new SaleReportDatasource(this.info.amounts);
    });
  }

  show() {
    const info = this.getInfo();
    this.router.navigate(['sale-report'], {
      queryParams: {
        startDate: info.startDate,
        finishDate: info.finishDate,
      },
    });
  }

  getInfo(): SaleReport {
    const formModel = this.form.value;

    return new SaleReport({
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
    });
  }

  print() {
    this.ser.print(this.info.startDate, this.info.finishDate).subscribe(
      () => {
        this.toaster.show('', 'Successfully Printed');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  exportCsv() {
    const headers = {
      Name: 'name',
      Amount: 'amount',
    };
    const csvData = new Blob([this.toCsv.toCsv(headers, this.dataSource.data)], {
      type: 'text/csv;charset=utf-8;',
    });
    const link = document.createElement('a');
    link.href = window.URL.createObjectURL(csvData);
    link.setAttribute('download', 'sale-report.csv');
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
}
